from iobeam.endpoints import service
from iobeam.http import request

'''
Communicates with the backend and exposes available Exports API methods.
'''
class ExportService(service.EndpointService):

    def __init__(self, token, requester=None):
        service.EndpointService.__init__(self, token, requester=requester)

    '''
    Wraps API call `POST /exports`

    Queries data from the iobeam backend.

    Params:
        query - A iobeam.resources.Query object that contains the parameters
                for the query.

    Returns:
        A dictionary representing the query results.

    Raises:
        Exception - If `query` is None
    '''
    def getData(self, query):
        if not self.token:
            raise request.UnauthorizedError.noTokenSet();
        if query is None:
            raise Exception("query cannot be None")

        endpoint = self.makeEndpoint("exports/{}".format(query.getUrl()))

        r = self.requester().get(endpoint).token(self.token)
        params = query.getParams()
        for p in params:
            r.setParam(p, params[p])
        r.execute()

        return r.getResponse()



from iobeam.utils import utils
from iobeam.resources import data

# alias query.TimeUnit to data.TimeUnit
TimeUnit = data.TimeUnit

'''
Represents a data query.
'''
class Query(object):

    @staticmethod
    def _validIntOrRaise(val, msg):
        if val is None:
            return False
        elif not isinstance(val, int):
            raise ValueError(msg)
        else:
            return True

    @staticmethod
    def _validPositiveIntOrRaise(val, msg):
        if not Query._validIntOrRaise(val, msg):
            return False
        elif val > 0:
            return True
        else:
            raise ValueError(msg)

    def __init__(self, projectId, deviceId=None, seriesName=None,
                 timeUnit=TimeUnit.MILLISECONDS):
        utils.checkValidProjectId(projectId)
        if timeUnit is None:
            timeUnit = TimeUnit.MILLISECONDS
        if not isinstance(timeUnit, TimeUnit):
            raise ValueError("timeUnit must be a query.TimeUnit")

        self._pid = projectId
        self._did = deviceId
        self._series = seriesName
        self._params = {}
        self._timeUnit = timeUnit
        if timeUnit != TimeUnit.MILLISECONDS:
            self._params["timefmt"] = str(timeUnit.value)

    '''
    Gets the resource locator portion of the exports API call

    Returns:
        String of format <projectId>/<deviceId or 'all'>/<seriesName or 'all'>
    '''
    def getUrl(self):
        did = self._did or "all"
        series = self._series or "all"
        return "{}/{}/{}".format(self._pid, did, series)

    def getParams(self):
        return self._params

    '''
    Sets the limit of this query, i.e., the max number of results series.
    '''
    def limit(self, limit):
        msg = "limit must be a positive int"
        if Query._validPositiveIntOrRaise(limit, msg):
            self._params["limit"] = limit
        return self

    '''
    Sets the from time limit for results.
    '''
    def fromTime(self, time):
        msg = "time must be an int ({} from epoch)".format(self._timeUnit.value)
        if Query._validIntOrRaise(time, msg):
            self._params["from"] = time
        return self

    '''
    Sets the to time limit for results.
    '''
    def toTime(self, time):
        msg = "time must be an int ({} from epoch)".format(self._timeUnit.value)
        if Query._validIntOrRaise(time, msg):
            self._params["to"] = time
        return self

    def inTimeRange(self, start, end):
        self.fromTime(start).toTime(end)
        if end < start:
            self._params.pop("from", None)
            self._params.pop("to", None)
            raise ValueError("end cannot be less than start")
        return self

    '''
    Sets the max value for values in the results.
    '''
    def lessThan(self, value):
        msg = "value must be an int"
        if Query._validIntOrRaise(value, msg):
            self._params["less_than"] = value
        return self

    '''
    Sets the min value for values in the results.
    '''
    def greaterThan(self, value):
        msg = "value must be an int"
        if Query._validIntOrRaise(value, msg):
            self._params["greater_than"] = value
        return self

    '''
    Sets the value that all results must equal.
    '''
    def equals(self, value):
        msg = "value must be an int"
        if Query._validIntOrRaise(value, msg):
            self._params["equals"] = value
        return self



'''
Common utility functions.
'''

# For compatibility with both Python 2 and 3.
try:
    unicode
except NameError as e:
    unicode = str

'''
Checks that a projectId is valid: a position int.

Raises:
    ValueError - If projectId is None, not an int, or <= 0.
'''
def checkValidProjectId(projectId):
    if projectId is None or not isinstance(projectId, int):
        raise ValueError("projectId must be an int")
    elif not projectId > 0:
        raise ValueError("projectId must be greater than 0")

def checkValidDeviceId(deviceId):
    if deviceId is None:
        raise ValueError("deviceId must be a string")
    elif not isinstance(deviceId, str) and not isinstance(deviceId, unicode):
        raise ValueError("deviceId must be a string")
    elif len(deviceId) == 0:
        raise ValueError("deviceId must be more than 0 characters")

def checkValidProjectToken(token):
    if token is None:
        raise ValueError("token must be a string")
    elif not isinstance(token, str) and not isinstance(token, unicode):
        raise ValueError("token must be a string")
    elif len(token) == 0:
        raise ValueError("token must be more than 0 characters")



from iobeam.endpoints import service
from iobeam.http import request
from iobeam.resources import device

'''
Communicates with the backend and exposes available Devices API methods.
'''
class DeviceService(service.EndpointService):

    def __init__(self, token, requester=None):
        service.EndpointService.__init__(self, token, requester=requester)

    '''
    Wraps API call `GET /devices/timestamp`.

    This returns the current time in milliseconds, according to the iobeam
    backend. Useful for clients with limited on-device clock support.

    Returns:
        Current timestamp in milliseconds since epoch; -1 if error.
    '''
    def getTimestamp(self):
        if not self.token:
            raise request.UnauthorizedError.noTokenSet();
        endpoint = self.makeEndpoint("devices/timestamp")

        r = self.requester().get(endpoint).token(self.token)
        r.execute()

        if r.getResponseCode() == 200:
            resp = r.getResponse()
            return resp["server_timestamp"]
        else:
            return -1

    '''
    Wraps API call `POST /devices`

    Registers the device in project `projectId` with the iobeam backend.
    `deviceId` and `deviceName` are optional, if provided they will used
    in registration, otherwise they will be generated by the backend.

    Params:
        projectId - Project ID to register device in
        deviceId - Desired device ID; if None, will be generated.
        deviceName - Desired device name; if None, will be generated.

    Returns:
        A Device object corresponding to the parameters (explicit and
        generated); None if there is an error/failure.
    '''
    def registerDevice(self, projectId, deviceId=None, deviceName=None):
        if not self.token:
            raise request.UnauthorizedError.noTokenSet();
        endpoint = self.makeEndpoint("devices")

        r = self.requester().post(endpoint).token(self.token)
        reqBody = {"project_id": projectId}
        if deviceId or deviceName:
            if deviceId:
                reqBody["device_id"] = deviceId
            if deviceName:
                reqBody["device_name"] = deviceName
        r.setBody(reqBody)
        r.execute()

        ret = None
        if r.getResponseCode() == 201:
            resp = r.getResponse()
            ret = device.Device(projectId, resp["device_id"],
                                deviceName=resp["device_name"])
        elif r.getResponseCode() == 403:
            raise request.UnauthorizedError("Invalid credentials.")
        else:
            raise request.Error("Received unexpected code: {}".format(
                r.getResponseCode()))

        return ret



import requests

class Requester(object):

    _BASE_URL = "https://api.iobeam.com/v1/"

    def __init__(self, baseUrl=_BASE_URL):
        self._baseUrl = baseUrl

    def makeEndpoint(self, endpoint):
        return self._baseUrl + endpoint

    def get(self, url):
        return Request("GET", url)

    def post(self, url):
        return Request("POST", url)


class UnauthorizedError(Exception):
    def __init__(self, value):
        Exception.__init__(self, value)

    @staticmethod
    def noTokenSet():
        return UnauthorizedError("no token set.")

class Error(Exception):
    def __init__(self, value):
        Exception.__init__(self, value)

class Request(object):

    def __init__(self, method, url):
        self.method = method
        self.url = url
        self.headers = {}
        self.header("User-Agent", "iobeam python")
        self.resp = None
        self.body = None
        self.params = {}

    def header(self, key, value):
        self.headers[key] = value
        return self

    def token(self, token):
        return self.header("Authorization", "Bearer {}".format(token))

    def setBody(self, body):
        self.body = body
        return self

    def setParam(self, key, value):
        self.params[key] = value
        return self

    def execute(self):
        self.resp = None
        if self.method == "GET":
            self.resp = requests.get(self.url, params=self.params, headers=self.headers)
        elif self.method == "POST":
            self.resp = requests.post(self.url, params=self.params, headers=self.headers, json=self.body)
        else:
            print("unsupported method")

    def getResponse(self):
        if self.resp is None:
            return None
        try:
            return self.resp.json()
        except:
            return None

    def getResponseCode(self):
        return self.resp.status_code



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import sys

install_requires = ['requests']
if "--python-tag" in sys.argv:
    i = 0
    while i < len(sys.argv):
        if sys.argv[i] == "--python-tag":
            break
        i += 1
    i += 1
    if i <= len(sys.argv):
        if sys.argv[i] != "py35":
            install_requires.append('enum34')

setup(
    name='iobeam',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.4.4',

    description='Library for connecting to iobeam, the data analysis \
    platform for the Internet of Things.',

    # The project's main homepage.
    url='https://github.com/iobeam/iobeam-client-python',

    # Author details
    author='iobeam',
    author_email='support@iobeam.com',

    # Choose your license
    license='Apache Software License',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],

    # What does your project relate to?
    keywords='iot data analytics iobeam sensors',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=install_requires,

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        #'dev': ['check-manifest'],
        #'test': ['coverage'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        #'sample': ['package_data.dat'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[],#('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    #entry_points={
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    #},
)



from iobeam.http import request

class EndpointService(object):

    def __init__(self, token, requester=None):
        self.token = token
        if requester is None:
            self._requester = request.Requester()
        else:
            self._requester = requester

    def requester(self):
        return self._requester

    def makeEndpoint(self, endpoint):
        return self._requester.makeEndpoint(endpoint)
